@@ -1,7 +1,5 @@
 /*
-  $Id: mmc2.cpp,v 1.3 2008/03/24 15:30:56 karl Exp $
-
-  Copyright (C) 2004, 2005, 2008 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2004, 2005, 2008, 2009 Rocky Bernstein <rocky@gnu.org>
   
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,13 +18,22 @@
 /* A program to using the MMC interface to list CD and drive features
    from the MMC GET_CONFIGURATION command . */
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+#include "config.h"
+#define __CDIO_CONFIG_H__ 1
 #endif
+
+#ifdef HAVE_STDIO_H
 #include <stdio.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
 #include <cdio/cdio.h>
 #include <cdio/mmc.h>
-#include <string.h>
 
 /* Set how long do wto wait for SCSI-MMC commands to complete */
 #define DEFAULT_TIMEOUT_MS 10000
