@@ -1,12 +1,10 @@
 /*
-    $Id: ds.h,v 1.3 2005/01/12 11:34:52 rocky Exp $
-
+    Copyright (C) 2005, 2008, 2017 Rocky Bernstein <rocky@gnu.org>
     Copyright (C) 2000, 2004 Herbert Valerio Riedel <hvr@gnu.org>
-    Copyright (C) 2005 Rocky Bernstein <rocky@panix.com>
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -15,20 +13,19 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/** \file ds.h 
+/** \file ds.h
  *  \brief  The top-level header for list-related data structures.
 
-    Note: this header will is slated to get removed and libcdio will use 
-    glib.h routines instead. 
+    Note: this header will is slated to get removed and libcdio will use
+    glib.h routines instead.
 */
 
 
-#ifndef __CDIO_DS_H__
-#define __CDIO_DS_H__
+#ifndef CDIO_DS_H_
+#define CDIO_DS_H_
 
 #include <cdio/types.h>
 
@@ -39,13 +36,6 @@ typedef struct _CdioListNode CdioListNode_t;
 typedef int (*_cdio_list_cmp_func_t) (void *p_data1, void *p_data2);
 typedef int (*_cdio_list_iterfunc_t) (void *p_data, void *p_user_data);
 
-/** The below are given compatibility with old code. Please use
-    the above type names, not these. */
-#define CdioList CdioList_t
-#define CdioListNode CdioListNode_t
-#define _cdio_list_cmp_func _cdio_list_cmp_func_t
-#define _cdio_list_iterfunc _cdio_list_iterfunc_t
-
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -61,11 +51,11 @@ void _cdio_list_prepend (CdioList_t *p_list, void *p_data);
 
 void _cdio_list_append (CdioList_t *p_list, void *p_data);
 
-void _cdio_list_foreach (CdioList_t *p_list, _cdio_list_iterfunc_t func, 
+void _cdio_list_foreach (CdioList_t *p_list, _cdio_list_iterfunc_t func,
                          void *p_user_data);
 
-CdioListNode_t *_cdio_list_find (CdioList_t *p_list, 
-                                 _cdio_list_iterfunc_t cmp_func, 
+CdioListNode_t *_cdio_list_find (CdioList_t *p_list,
+                                 _cdio_list_iterfunc_t cmp_func,
                                  void *p_user_data);
 
 #define _CDIO_LIST_FOREACH(node, list) \
@@ -73,11 +63,11 @@ CdioListNode_t *_cdio_list_find (CdioList_t *p_list,
 
 /** node operations */
 
-CdioListNode *_cdio_list_begin (const CdioList_t *p_list);
+CdioListNode_t *_cdio_list_begin (const CdioList_t *p_list);
 
-CdioListNode *_cdio_list_end (CdioList_t *p_list);
+CdioListNode_t *_cdio_list_end (CdioList_t *p_list);
 
-CdioListNode *_cdio_list_node_next (CdioListNode_t *p_node);
+CdioListNode_t *_cdio_list_node_next (CdioListNode_t *p_node);
 
 void _cdio_list_node_free (CdioListNode_t *p_node, int i_free_data);
 
@@ -87,13 +77,12 @@ void *_cdio_list_node_data (CdioListNode_t *p_node);
 }
 #endif /* __cplusplus */
 
-#endif /* __CDIO_DS_H__ */
+#endif /* CDIO_DS_H_ */
 
-/* 
+/*
  * Local variables:
  *  c-file-style: "gnu"
  *  tab-width: 8
  *  indent-tabs-mode: nil
  * End:
  */
-
