@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003, 2004, 2005, 2008, 2009, 2010, 2011
+  Copyright (C) 2003, 2004-2005, 2008-2011, 2014, 2017
   Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
@@ -53,13 +53,13 @@
 
 static lba_t get_track_lba_freebsd(void *p_user_data, track_t i_track);
 
-static access_mode_t 
-str_to_access_mode_freebsd(const char *psz_access_mode) 
+static access_mode_t
+str_to_access_mode_freebsd(const char *psz_access_mode)
 {
   const access_mode_t default_access_mode = DEFAULT_FREEBSD_AM;
 
   if (NULL==psz_access_mode) return default_access_mode;
-  
+
   if (!strcmp(psz_access_mode, "ioctl"))
     return _AM_IOCTL;
   else if (!strcmp(psz_access_mode, "CAM"))
@@ -69,7 +69,7 @@ str_to_access_mode_freebsd(const char *psz_access_mode)
   else if (!strcmp(psz_access_mode, "MMC_RDWR_EXCL"))
     return _AM_MMC_RDWR_EXCL;
   else {
-    cdio_warn ("unknown access type: %s. Default used.", 
+    cdio_warn ("unknown access type: %s. Default used.",
 	       psz_access_mode);
     return default_access_mode;
   }
@@ -98,7 +98,7 @@ free_freebsd (void *p_obj)
   }
 }
 
-/* Check a drive to see if it is a CD-ROM 
+/* Check a drive to see if it is a CD-ROM
    Return 1 if a CD-ROM. 0 if it exists but isn't a CD-ROM drive
    and -1 if no device exists .
 */
@@ -110,7 +110,7 @@ cdio_is_cdrom(char *drive, char *mnttype)
 
 /*!
    Reads i_blocks of audio sectors from cd device into data starting from lsn.
-   Returns 0 if no error. 
+   Returns 0 if no error.
  */
 static driver_return_code_t
 read_audio_sectors_freebsd (void *p_user_data, void *p_buf, lsn_t i_lsn,
@@ -124,7 +124,7 @@ read_audio_sectors_freebsd (void *p_user_data, void *p_buf, lsn_t i_lsn,
       return mmc_read_sectors( p_env->gen.cdio, p_buf, i_lsn,
                                   CDIO_MMC_READ_TYPE_CDDA, i_blocks);
     case _AM_IOCTL:
-      return read_audio_sectors_freebsd_ioctl(p_user_data, p_buf, i_lsn, 
+      return read_audio_sectors_freebsd_ioctl(p_user_data, p_buf, i_lsn,
 					      i_blocks);
     case _AM_NONE:
       cdio_info ("access mode not set");
@@ -135,7 +135,7 @@ read_audio_sectors_freebsd (void *p_user_data, void *p_buf, lsn_t i_lsn,
 
 /*!
    Reads a single mode2 sector from cd device into data starting
-   from i_lsn. Returns 0 if no error. 
+   from i_lsn. Returns 0 if no error.
  */
 static driver_return_code_t
 read_mode2_sector_freebsd (void *p_user_data, void *data, lsn_t i_lsn,
@@ -176,11 +176,11 @@ read_mode2_sectors_freebsd (void *p_user_data, void *p_data, lsn_t i_lsn,
   } else {
     unsigned int i;
     uint16_t i_blocksize = b_form2 ? M2RAW_SECTOR_SIZE : CDIO_CD_FRAMESIZE;
-  
+
     /* For each frame, pick out the data part we need */
     for (i = 0; i < i_blocks; i++) {
-      int retval = read_mode2_sector_freebsd (p_env, 
-					       ((char *)p_data) + 
+      int retval = read_mode2_sector_freebsd (p_env,
+					       ((char *)p_data) +
 					       (i_blocksize * i),
 					       i_lsn + i, b_form2);
       if (retval) return retval;
@@ -193,7 +193,7 @@ read_mode2_sectors_freebsd (void *p_user_data, void *p_data, lsn_t i_lsn,
    Return the size of the CD in logical block address (LBA) units.
   @return the lsn. On error return CDIO_INVALID_LSN.
  */
-static lsn_t 
+static lsn_t
 get_disc_last_lsn_freebsd (void *p_obj)
 {
   _img_private_t *p_env = p_obj;
@@ -217,10 +217,10 @@ get_disc_last_lsn_freebsd (void *p_obj)
 /*!
   Set the arg "key" with "value" in the source device.
   Currently "source" and "access-mode" are valid keys.
-  "source" sets the source device in I/O operations 
-  "access-mode" sets the the method of CD access 
+  "source" sets the source device in I/O operations
+  "access-mode" sets the the method of CD access
 
-  DRIVER_OP_SUCCESS is returned if no error was found, 
+  DRIVER_OP_SUCCESS is returned if no error was found,
   and nonzero if there as an error.
 */
 static driver_return_code_t
@@ -237,8 +237,8 @@ set_arg_freebsd (void *p_user_data, const char key[], const char value[])
   else if (!strcmp (key, "access-mode"))
     {
       p_env->access_mode = str_to_access_mode_freebsd(value);
-      if (p_env->access_mode == _AM_CAM && !p_env->b_cam_init) 
-	return init_freebsd_cam(p_env) 
+      if (p_env->access_mode == _AM_CAM && !p_env->b_cam_init)
+	return init_freebsd_cam(p_env)
 	  ? DRIVER_OP_SUCCESS : DRIVER_OP_ERROR;
     }
   else return DRIVER_OP_ERROR;
@@ -248,7 +248,7 @@ set_arg_freebsd (void *p_user_data, const char key[], const char value[])
 }
 
 /* Set CD-ROM drive speed */
-static int 
+static int
 set_speed_freebsd (void *p_user_data, int i_speed)
 {
   const _img_private_t *p_env = p_user_data;
@@ -262,12 +262,12 @@ set_speed_freebsd (void *p_user_data, int i_speed)
 #endif
 }
 
-/*! 
+/*!
   Read and cache the CD's Track Table of Contents and track info.
   Return false if unsuccessful;
 */
 static bool
-read_toc_freebsd (void *p_user_data) 
+read_toc_freebsd (void *p_user_data)
 {
   _img_private_t *p_env = p_user_data;
   track_t i, j;
@@ -279,19 +279,19 @@ read_toc_freebsd (void *p_user_data)
   }
 
   p_env->gen.i_first_track = p_env->tochdr.starting_track;
-  p_env->gen.i_tracks      = p_env->tochdr.ending_track - 
+  p_env->gen.i_tracks      = p_env->tochdr.ending_track -
     p_env->gen.i_first_track + 1;
 
   j=0;
   for (i=p_env->gen.i_first_track; i<=p_env->gen.i_tracks; i++, j++) {
-    struct ioc_read_toc_single_entry *p_toc = 
+    struct ioc_read_toc_single_entry *p_toc =
       &(p_env->tocent[i-p_env->gen.i_first_track]);
     p_toc->track = i;
     p_toc->address_format = CD_LBA_FORMAT;
 
     if ( ioctl(p_env->gen.fd, CDIOREADTOCENTRY, p_toc) ) {
       cdio_warn("%s %d: %s\n",
-		 "error in ioctl CDROMREADTOCENTRY for track", 
+		 "error in ioctl CDROMREADTOCENTRY for track",
 		 i, strerror(errno));
       return false;
     }
@@ -304,7 +304,7 @@ read_toc_freebsd (void *p_user_data)
   p_env->tocent[j].address_format = CD_LBA_FORMAT;
   if ( ioctl(p_env->gen.fd, CDIOREADTOCENTRY, &(p_env->tocent[j]) ) ){
     cdio_warn("%s: %s\n",
-	       "error in ioctl CDROMREADTOCENTRY for leadout track", 
+	       "error in ioctl CDROMREADTOCENTRY for leadout track",
 	       strerror(errno));
     return false;
   }
@@ -329,7 +329,7 @@ audio_get_volume_freebsd (void *p_user_data,
 
 /*!
   Pause playing CD through analog output
-  
+
   @param p_cdio the CD object to be acted upon.
 */
 static driver_return_code_t
@@ -342,11 +342,11 @@ audio_pause_freebsd (void *p_user_data)
 
 /*!
   Playing starting at given MSF through analog output
-  
+
   @param p_cdio the CD object to be acted upon.
 */
 static driver_return_code_t
-audio_play_msf_freebsd (void *p_user_data, msf_t *p_start_msf, 
+audio_play_msf_freebsd (void *p_user_data, msf_t *p_start_msf,
 			msf_t *p_end_msf)
 {
   const _img_private_t *p_env = p_user_data;
@@ -365,19 +365,19 @@ audio_play_msf_freebsd (void *p_user_data, msf_t *p_start_msf,
 
 /*!
   Playing CD through analog output at the desired track and index
-  
+
   @param p_user_data the CD object to be acted upon.
   @param p_track_index location to start/end.
 */
 static driver_return_code_t
-audio_play_track_index_freebsd (void *p_user_data, 
+audio_play_track_index_freebsd (void *p_user_data,
 				cdio_track_index_t *p_track_index)
 {
   const _img_private_t *p_env = p_user_data;
   msf_t start_msf;
   msf_t end_msf;
   struct ioc_play_msf freebsd_play_msf;
-  lsn_t i_lsn = get_track_lba_freebsd(p_user_data, 
+  lsn_t i_lsn = get_track_lba_freebsd(p_user_data,
 				      p_track_index->i_start_track);
 
   cdio_lsn_to_msf(i_lsn, &start_msf);
@@ -398,13 +398,13 @@ audio_play_track_index_freebsd (void *p_user_data,
 
 /*!
   Read Audio Subchannel information
-  
+
   @param p_user_data the CD object to be acted upon.
   @param p_subchannel returned information
 */
 #if 1
 static driver_return_code_t
-audio_read_subchannel_freebsd (void *p_user_data, 
+audio_read_subchannel_freebsd (void *p_user_data,
 			       /*out*/ cdio_subchannel_t *p_subchannel)
 {
   const _img_private_t *p_env = p_user_data;
@@ -439,9 +439,9 @@ audio_read_subchannel_freebsd (void *p_user_data,
 
 /*!
   Resume playing an audio CD.
-  
+
   @param p_cdio the CD object to be acted upon.
-  
+
 */
 static driver_return_code_t
 audio_resume_freebsd (void *p_user_data)
@@ -452,12 +452,12 @@ audio_resume_freebsd (void *p_user_data)
 
 /*!
   Set the volume of an audio CD.
-  
+
   @param p_cdio the CD object to be acted upon.
-  
+
 */
 static driver_return_code_t
-audio_set_volume_freebsd (void *p_user_data, 
+audio_set_volume_freebsd (void *p_user_data,
 			  cdio_audio_volume_t *p_volume)
 {
   const _img_private_t *p_env = p_user_data;
@@ -467,8 +467,8 @@ audio_set_volume_freebsd (void *p_user_data,
 /*!
   Eject media. Return 1 if successful, 0 otherwise.
  */
-static int 
-eject_media_freebsd (void *p_user_data) 
+static int
+eject_media_freebsd (void *p_user_data)
 {
   _img_private_t *p_env = p_user_data;
 
@@ -488,11 +488,11 @@ eject_media_freebsd (void *p_user_data)
 
 /*!
   Stop playing an audio CD.
-  
+
   @param p_user_data the CD object to be acted upon.
-  
+
 */
-static driver_return_code_t 
+static driver_return_code_t
 audio_stop_freebsd (void *p_user_data)
 {
   const _img_private_t *p_env = p_user_data;
@@ -514,10 +514,10 @@ audio_stop_freebsd (void *p_user_data)
 */
 static int
 set_scsi_tuple_freebsd(_img_private_t *env)
-{   
+{
   int bus_no = -1, host_no = -1, channel_no = -1, target_no = -1, lun_no = -1;
   int ret;
-  char tuple[160]; 
+  char tuple[160];
 
   ret = obtain_scsi_adr_freebsd_cam(env->gen.source_name,
                                    &bus_no, &host_no, &channel_no,
@@ -535,8 +535,8 @@ set_scsi_tuple_freebsd(_img_private_t *env)
   sprintf(tuple, "%d,%d,%d,%d,%d",
           bus_no, host_no, channel_no, target_no, lun_no);
   env->gen.scsi_tuple = strdup(tuple);
-  return 1; 
-} 
+  return 1;
+}
 
 static bool
 is_mmc_supported(void *user_data)
@@ -584,8 +584,8 @@ get_arg_freebsd (void *user_data, const char key[])
       set_scsi_tuple_freebsd(env);
     return env->gen.scsi_tuple;
   } else if (!strcmp (key, "mmc-supported?")) {
-      is_mmc_supported(user_data) ? "true" : "false";
-  } 
+    return is_mmc_supported(user_data) ? "true" : "false";
+  }
   return NULL;
 }
 
@@ -595,7 +595,7 @@ get_arg_freebsd (void *user_data, const char key[])
   Note: string is malloc'd so caller should free() then returned
   string when done with it.
 
-  FIXME: This is just a guess. 
+  FIXME: This is just a guess.
 
  */
 static char *
@@ -616,11 +616,37 @@ get_mcn_freebsd (const void *p_user_data) {
   return NULL;
 }
 
+/*!
+  Return the international standard recording code ISRC.
+
+  Note: string is malloc'd so caller should free() then returned
+  string when done with it.
+
+ */
+static char *
+get_track_isrc_freebsd (const void *p_user_data,
+			track_t i_track) {
+  const _img_private_t *p_env = p_user_data;
+
+  switch (p_env->access_mode) {
+    case _AM_CAM:
+    case _AM_MMC_RDWR:
+    case _AM_MMC_RDWR_EXCL:
+      return mmc_get_track_isrc(p_env->gen.cdio, i_track);
+    case _AM_IOCTL:
+      return mmc_get_track_isrc(p_env->gen.cdio, i_track);
+    case _AM_NONE:
+      cdio_info ("access mode not set");
+      return NULL;
+  }
+  return NULL;
+}
+
 static void
 get_drive_cap_freebsd (const void *p_user_data,
 		       cdio_drive_read_cap_t  *p_read_cap,
 		       cdio_drive_write_cap_t *p_write_cap,
-		       cdio_drive_misc_cap_t  *p_misc_cap) 
+		       cdio_drive_misc_cap_t  *p_misc_cap)
 {
   const _img_private_t *p_env = p_user_data;
 
@@ -636,26 +662,26 @@ get_drive_cap_freebsd (const void *p_user_data,
       cdio_info ("access mode not set");
       return;
   }
-}  
+}
 
 /*!
-  Run a SCSI MMC command. 
- 
+  Run a SCSI MMC command.
+
   p_user_data   internal CD structure.
   i_timeout_ms  time in milliseconds we will wait for the command
-                to complete. If this value is -1, use the default 
+                to complete. If this value is -1, use the default
 		time-out value.
   i_cdb	        Size of p_cdb
-  p_cdb	        CDB bytes. 
+  p_cdb	        CDB bytes.
   e_direction	direction the transfer is to go.
   i_buf	        Size of buffer
   p_buf	        Buffer for data, both sending and receiving
  */
 static driver_return_code_t
 run_mmc_cmd_freebsd( void *p_user_data, unsigned int i_timeout_ms,
-		     unsigned int i_cdb, const mmc_cdb_t *p_cdb, 
-		     cdio_mmc_direction_t e_direction, 
-		     unsigned int i_buf, /*in/out*/ void *p_buf ) 
+		     unsigned int i_cdb, const mmc_cdb_t *p_cdb,
+		     cdio_mmc_direction_t e_direction,
+		     unsigned int i_buf, /*in/out*/ void *p_buf )
 {
   const _img_private_t *p_env = p_user_data;
   int ret;
@@ -664,7 +690,7 @@ run_mmc_cmd_freebsd( void *p_user_data, unsigned int i_timeout_ms,
     case _AM_CAM:
     case _AM_MMC_RDWR:
     case _AM_MMC_RDWR_EXCL:
-      ret = run_mmc_cmd_freebsd_cam( p_user_data, i_timeout_ms, i_cdb, p_cdb, 
+      ret = run_mmc_cmd_freebsd_cam( p_user_data, i_timeout_ms, i_cdb, p_cdb,
                                      e_direction, i_buf, p_buf );
       if (ret != 0)
         return DRIVER_OP_ERROR;
@@ -676,16 +702,16 @@ run_mmc_cmd_freebsd( void *p_user_data, unsigned int i_timeout_ms,
       return DRIVER_OP_ERROR;
   }
   return DRIVER_OP_ERROR;
-}  
+}
 
-/*!  
-  Get format of track. 
+/*!
+  Get format of track.
 
   FIXME: We're just guessing this from the GNU/Linux code.
-  
+
 */
 static track_format_t
-get_track_format_freebsd(void *p_user_data, track_t i_track) 
+get_track_format_freebsd(void *p_user_data, track_t i_track)
 {
   _img_private_t *p_env = p_user_data;
 
@@ -708,7 +734,7 @@ get_track_format_freebsd(void *p_user_data, track_t i_track)
       return TRACK_FORMAT_DATA;
   } else
     return TRACK_FORMAT_AUDIO;
-  
+
 }
 
 /*!
@@ -720,22 +746,22 @@ get_track_format_freebsd(void *p_user_data, track_t i_track)
   FIXME: there's gotta be a better design for this and get_track_format?
 */
 static bool
-get_track_green_freebsd(void *user_data, track_t i_track) 
+get_track_green_freebsd(void *user_data, track_t i_track)
 {
   _img_private_t *p_env = user_data;
-  
+
   if (i_track == CDIO_CDROM_LEADOUT_TRACK) i_track = TOTAL_TRACKS+1;
 
   if (i_track > TOTAL_TRACKS+1 || i_track == 0)
     return false;
 
-  /* FIXME: Dunno if this is the right way, but it's what 
+  /* FIXME: Dunno if this is the right way, but it's what
      I was using in cdinfo for a while.
    */
   return ((p_env->tocent[i_track-FIRST_TRACK_NUM].entry.control & 2) != 0);
 }
 
-/*!  
+/*!
   Return the starting LSN track number
   i_track in obj.  Track numbers start at 1.
   The "leadout" track is specified either by
@@ -772,9 +798,9 @@ cdio_get_devices_freebsd (void)
   char drive[40];
   char **drives = NULL;
   unsigned int num_drives=0;
-  bool exists = true, have_cam_drive = false;
+  bool exists = true;
   char c;
-  
+
   /* Scan the system for CD-ROM drives.
   */
 
@@ -782,14 +808,14 @@ cdio_get_devices_freebsd (void)
 
   struct fstab *fs;
   setfsent();
-  
+
   /* Check what's in /etc/fstab... */
   while ( (fs = getfsent()) )
     {
       if (strncmp(fs->fs_spec, "/dev/sr", 7))
 	cdio_add_device_list(&drives, fs->fs_spec, &num_drives);
     }
-  
+
 #endif
 
   /* Scan the system for CD-ROM drives.
@@ -803,13 +829,12 @@ cdio_get_devices_freebsd (void)
     exists = cdio_is_cdrom(drive, NULL);
     if ( exists ) {
       cdio_add_device_list(&drives, drive, &num_drives);
-      have_cam_drive = true;
     }
   }
 
   /* Scan ATAPI devices */
- 
-  /* ??? ts 9 Jan 2009 
+
+  /* ??? ts 9 Jan 2009
      For now i assume atapicam running if a cdN device was found.
      man atapicam strongly discourages to mix usage of CAM and ATAPI device.
      So on the risk to sabotage systems without atapicam but with real old
@@ -822,8 +847,6 @@ cdio_get_devices_freebsd (void)
      whether atapicam runs, or to make pairs of cd and acd.
 
   */
-  exists = true;
-  
   for ( c='0'; exists && c <='9'; c++ ) {
     sprintf(drive, "/dev/acd%c%s", c, DEVICE_POSTFIX);
     exists = cdio_is_cdrom(drive, NULL);
@@ -848,7 +871,7 @@ cdio_get_default_device_freebsd()
   char drive[40];
   bool exists=true;
   char c;
-  
+
   /* Scan the system for CD-ROM drives.
   */
 
@@ -856,14 +879,14 @@ cdio_get_default_device_freebsd()
 
   struct fstab *fs;
   setfsent();
-  
+
   /* Check what's in /etc/fstab... */
   while ( (fs = getfsent()) )
     {
       if (strncmp(fs->fs_spec, "/dev/sr", 7))
 	return strdup(fs->fs_spec);
     }
-  
+
 #endif
 
   /* Scan the system for CD-ROM drives.
@@ -881,7 +904,7 @@ cdio_get_default_device_freebsd()
 
   /* Scan are ATAPI devices */
   exists = true;
-  
+
   for ( c='0'; exists && c <='9'; c++ ) {
     sprintf(drive, "/dev/acd%c%s", c, DEVICE_POSTFIX);
     exists = cdio_is_cdrom(drive, NULL);
@@ -895,25 +918,25 @@ cdio_get_default_device_freebsd()
 
 /*!
   Close tray on CD-ROM.
-  
+
   @param psz_device the CD-ROM drive to be closed.
-  
+
 */
-driver_return_code_t 
+driver_return_code_t
 close_tray_freebsd (const char *psz_device)
 {
 #ifdef HAVE_FREEBSD_CDROM
   int fd = open (psz_device, O_RDONLY|O_NONBLOCK, 0);
   int i_rc;
-  
+
   if((i_rc = ioctl(fd, CDIOCCLOSE)) != 0) {
-    cdio_warn ("ioctl CDIOCCLOSE failed: %s\n", strerror(errno));  
+    cdio_warn ("ioctl CDIOCCLOSE failed: %s\n", strerror(errno));
     close(fd);
     return DRIVER_OP_ERROR;
   }
   close(fd);
   return DRIVER_OP_SUCCESS;
-#else 
+#else
   return DRIVER_OP_NO_DRIVER;
 #endif /*HAVE_FREEBSD_CDROM*/
 }
@@ -923,20 +946,20 @@ close_tray_freebsd (const char *psz_device)
   @return 1 if media has changed since last call, 0 if not. Error
   return codes are the same as driver_return_code_t
    */
-   
+
 #ifdef HAVE_FREEBSD_CDROM
 int
 get_media_changed_freebsd (const void *p_user_data)
 {
   const _img_private_t *p_env = p_user_data;
   int changed = 0 ;
-  
+
   changed = mmc_get_media_changed( p_env->gen.cdio );
-    
+
   return ((changed > 0) ? changed : 0);
 }
 
-static const char* 
+static const char*
 get_access_mode(const char *psz_source)
 {
     char *psz_src;
@@ -947,15 +970,15 @@ get_access_mode(const char *psz_source)
     }
 
     if (psz_src) {
-	if (!(strncmp(psz_src, "/dev/acd", 8))) 
+	if (!(strncmp(psz_src, "/dev/acd", 8)))
 	    return "ioctl";
 	else {
 	    char devname[256];
 	    int  bytes = readlink(psz_src, devname, 255);
-	    
+
 	    if (bytes > 0) {
 		devname[bytes]=0;
-		if (!(strncmp(devname, "acd", 3))) 
+		if (!(strncmp(devname, "acd", 3)))
 		    return "ioctl";
 	    }
 	}
@@ -992,7 +1015,7 @@ static int freebsd_dev_lock(int dev_fd, char *devname,
       sprintf(pass_name, "/dev/pass%d", i);
       if (stat(pass_name, &name_stbuf) != -1)
         if(fd_stbuf.st_ino == name_stbuf.st_ino &&
-           fd_stbuf.st_dev == name_stbuf.st_dev)  
+           fd_stbuf.st_dev == name_stbuf.st_dev)
     break;
     }
     if (i < pass_l) {
@@ -1024,7 +1047,7 @@ static int freebsd_dev_lock(int dev_fd, char *devname,
   }
 
 /*
-  fprintf(stderr, "libburn_DEBUG: flock obtained on %s of %s\n", 
+  fprintf(stderr, "libburn_DEBUG: flock obtained on %s of %s\n",
                   lock_name, devname);
 */
 
@@ -1084,7 +1107,7 @@ cdio_open_freebsd (const char *psz_source_name)
   ones to set that up.
  */
 CdIo *
-cdio_open_am_freebsd (const char *psz_orig_source_name, 
+cdio_open_am_freebsd (const char *psz_orig_source_name,
 		      const char *psz_access_mode)
 {
 
@@ -1093,9 +1116,6 @@ cdio_open_am_freebsd (const char *psz_orig_source_name,
   _img_private_t *_data;
   char *psz_source_name;
   int open_access_mode;  /* Access mode passed to cdio_generic_init. */
-  
-  if (!psz_access_mode) 
-      psz_access_mode = get_access_mode(psz_orig_source_name);
 
   cdio_funcs_t _funcs = {
     .audio_get_volume       = audio_get_volume_freebsd,
@@ -1125,9 +1145,10 @@ cdio_open_am_freebsd (const char *psz_orig_source_name,
     .get_track_copy_permit  = get_track_copy_permit_generic,
     .get_track_format       = get_track_format_freebsd,
     .get_track_green        = get_track_green_freebsd,
-    .get_track_lba          = get_track_lba_freebsd, 
+    .get_track_lba          = get_track_lba_freebsd,
     .get_track_preemphasis  = get_track_preemphasis_generic,
     .get_track_msf          = NULL,
+    .get_track_isrc         = get_track_isrc_freebsd,
     .lseek                  = cdio_generic_lseek,
     .read                   = cdio_generic_read,
     .read_audio_sectors     = read_audio_sectors_freebsd,
@@ -1141,6 +1162,9 @@ cdio_open_am_freebsd (const char *psz_orig_source_name,
     .set_speed              = set_speed_freebsd,
   };
 
+  if (!psz_access_mode)
+      psz_access_mode = get_access_mode(psz_orig_source_name);
+
   _data                     = calloc(1, sizeof (_img_private_t));
   _data->access_mode        = str_to_access_mode_freebsd(psz_access_mode);
   _data->gen.init           = false;
@@ -1166,7 +1190,7 @@ cdio_open_am_freebsd (const char *psz_orig_source_name,
       return NULL;
     }
   }
-    
+
   ret = cdio_new ((void *)_data, &_funcs);
   if (ret == NULL) return NULL;
 
@@ -1206,7 +1230,7 @@ cdio_open_am_freebsd (const char *psz_orig_source_name,
     if ( _data->access_mode == _AM_IOCTL ) {
       return ret;
     } else {
-      if (init_freebsd_cam(_data)) 
+      if (init_freebsd_cam(_data))
 	return ret;
       else {
 	cdio_generic_free (_data);
@@ -1217,8 +1241,8 @@ cdio_open_am_freebsd (const char *psz_orig_source_name,
     cdio_generic_free (_data);
     return NULL;
   }
-  
-#else 
+
+#else
   return NULL;
 #endif /* HAVE_FREEBSD_CDROM */
 
@@ -1229,7 +1253,7 @@ cdio_have_freebsd (void)
 {
 #ifdef HAVE_FREEBSD_CDROM
   return true;
-#else 
+#else
   return false;
 #endif /* HAVE_FREEBSD_CDROM */
 }
