@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2003, 2004, 2005, 2008, 2009, 2011, 2012
+  Copyright (C) 2003-2005, 2008-2009, 2011-2012, 2016-2017
   Rocky Bernstein <rocky@gnu.org>
 
   This program is free software: you can redistribute it and/or modify
@@ -26,6 +26,13 @@
 # include "config.h"
 #endif
 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+
 #include <cdio/cdio.h>
 #include <cdio/audio.h>
 #include <cdio/cdtext.h>
@@ -35,8 +42,34 @@
 extern "C" {
 #endif /* __cplusplus */
 
-  extern const char * cdio_dirname(const char *fname);
-  extern const char *cdio_abspath(const char *cwd, const char *fname);
+#ifndef HAVE_STRNDUP
+static inline char *strndup(const char *s, size_t n)
+{
+    char *result;
+    size_t len = strlen (s);
+    if (n < len)
+        len = n;
+    result = (char *) malloc (len + 1);
+    if (!result)
+        return 0;
+    result[len] = '\0';
+    return (char *) strncpy (result, s, len);
+}
+#endif /*HAVE_STRNDUP*/
+
+  /*!
+    Get directory name from file name.
+
+    Callers must free return value after use.
+   */
+  extern char *cdio_dirname(const char *fname);
+
+  /*!
+    Construct an absolute file name from path and file name.
+
+    Callers must free return value after use.
+   */
+  extern char *cdio_abspath(const char *cwd, const char *fname);
 
   /* Opaque type */
   typedef struct _CdioDataSource CdioDataSource_t;
@@ -54,101 +87,101 @@ extern "C" {
 
 
   typedef struct {
-    
+
     /*!
       Get volume of an audio CD.
-      
+
       @param p_env the CD object to be acted upon.
-      
+
     */
-    driver_return_code_t (*audio_get_volume) 
+    driver_return_code_t (*audio_get_volume)
          (void *p_env,  /*out*/ cdio_audio_volume_t *p_volume);
 
     /*!
       Pause playing CD through analog output
-      
+
       @param p_env the CD object to be acted upon.
     */
     driver_return_code_t (*audio_pause) (void *p_env);
-    
+
     /*!
       Playing CD through analog output
-      
+
       @param p_env the CD object to be acted upon.
     */
-    driver_return_code_t (*audio_play_msf) ( void *p_env, 
+    driver_return_code_t (*audio_play_msf) ( void *p_env,
                                              msf_t *p_start_msf,
                                              msf_t *p_end_msf );
 
     /*!
       Playing CD through analog output
-      
+
       @param p_env the CD object to be acted upon.
     */
-    driver_return_code_t (*audio_play_track_index) 
+    driver_return_code_t (*audio_play_track_index)
          ( void *p_env, cdio_track_index_t *p_track_index );
 
     /*!
       Get subchannel information.
-      
+
       @param p_env the CD object to be acted upon.
     */
-    driver_return_code_t (*audio_read_subchannel) 
+    driver_return_code_t (*audio_read_subchannel)
          ( void *p_env, cdio_subchannel_t *subchannel );
 
     /*!
       Resume playing an audio CD.
-      
+
       @param p_env the CD object to be acted upon.
-      
+
     */
     driver_return_code_t (*audio_resume) ( void *p_env );
-    
+
     /*!
       Set volume of an audio CD.
-      
+
       @param p_env the CD object to be acted upon.
-      
+
     */
-    driver_return_code_t (*audio_set_volume) 
+    driver_return_code_t (*audio_set_volume)
          ( void *p_env,  cdio_audio_volume_t *p_volume );
 
     /*!
       Stop playing an audio CD.
-      
+
       @param p_env the CD object to be acted upon.
-      
+
     */
     driver_return_code_t (*audio_stop) ( void *p_env );
-    
+
     /*!
-      Eject media in CD drive. If successful, as a side effect we 
-      also free p_env. 
+      Eject media in CD drive. If successful, as a side effect we
+      also free p_env.
 
       @param p_env the CD object to be acted upon.
       If the CD is ejected *p_env is freed and p_env set to NULL.
     */
     driver_return_code_t (*eject_media) ( void *p_env );
-    
+
     /*!
-      Release and free resources associated with cd. 
+      Release and free resources associated with cd.
     */
     void (*free) (void *p_env);
-    
+
     /*!
       Return the value associated with the key "arg".
     */
     const char * (*get_arg) (void *p_env, const char key[]);
-    
+
     /*!
-      Get the block size for subsequest read requests, via a SCSI MMC 
+      Get the block size for subsequest read requests, via a SCSI MMC
       MODE_SENSE 6 command.
     */
     int (*get_blocksize) ( void *p_env );
-    
-    /*! 
+
+    /*!
       Get cdtext information for a CdIo object.
-    
+
       @param obj the CD object that may contain CD-TEXT information.
       @return the CD-TEXT object or NULL if obj is NULL
       or CD-TEXT information does not exist.
@@ -165,16 +198,16 @@ extern "C" {
       free when done and not NULL.
     */
     uint8_t * (*get_cdtext_raw) ( void *p_env );
-    
+
     /*!
       Return an array of device names. if CdIo is NULL (we haven't
-      initialized a specific device driver), then find a suitable device 
+      initialized a specific device driver), then find a suitable device
       driver.
-      
+
       NULL is returned if we couldn't return a list of devices.
     */
     char ** (*get_devices) ( void );
-    
+
     /*!
       Get the default CD device.
 
@@ -186,21 +219,21 @@ extern "C" {
       NULL even though there may be a hardware CD-ROM.
     */
     char * (*get_default_device) ( void );
-    
+
     /*!
       Return the size of the CD in logical block address (LBA) units.
       @return the lsn. On error 0 or CDIO_INVALD_LSN.
     */
     lsn_t (*get_disc_last_lsn) ( void *p_env );
 
-    /*! 
+    /*!
       Get disc mode associated with cd_obj.
     */
     discmode_t (*get_discmode) ( void *p_env );
 
     /*!
       Return the what kind of device we've got.
-      
+
       See cd_types.h for a list of bitmasks for the drive type;
     */
     void (*get_drive_cap) (const void *p_env,
@@ -208,28 +241,28 @@ extern "C" {
                            cdio_drive_write_cap_t *p_write_cap,
                            cdio_drive_misc_cap_t  *p_misc_cap);
     /*!
-      Return the number of of the first track. 
+      Return the number of of the first track.
       CDIO_INVALID_TRACK is returned on error.
     */
     track_t (*get_first_track_num) ( void *p_env );
-    
-    /*! 
+
+    /*!
       Get the CD-ROM hardware info via a SCSI MMC INQUIRY command.
       False is returned if we had an error getting the information.
     */
-    bool (*get_hwinfo) 
+    bool (*get_hwinfo)
          ( const CdIo_t *p_cdio, /* out*/ cdio_hwinfo_t *p_hw_info );
 
     /*! Get the LSN of the first track of the last session of
       on the CD.
-           
+
        @param p_cdio the CD object to be acted upon.
        @param i_last_session pointer to the session number to be returned.
     */
     driver_return_code_t (*get_last_session)
          ( void *p_env, /*out*/ lsn_t *i_last_session );
 
-    /*! 
+    /*!
       Find out if media has changed since the last call.
       @param p_env the CD object to be acted upon.
       @return 1 if media has changed since last call, 0 if not. Error
@@ -237,31 +270,31 @@ extern "C" {
     */
     int (*get_media_changed) ( const void *p_env );
 
-    /*!  
+    /*!
       Return the media catalog number MCN from the CD or NULL if
       there is none or we don't have the ability to get it.
     */
     char * (*get_mcn) ( const void *p_env );
 
-    /*! 
+    /*!
       Return the number of tracks in the current medium.
       CDIO_INVALID_TRACK is returned on error.
     */
     track_t (*get_num_tracks) ( void *p_env );
-    
+
     /*! Return number of channels in track: 2 or 4; -2 if not
       implemented or -1 for error.
       Not meaningful if track is not an audio track.
     */
     int (*get_track_channels) ( const void *p_env, track_t i_track );
-  
+
     /*! Return 0 if track is copy protected, 1 if not, or -1 for error
       or -2 if not implimented (yet). Is this meaningful if not an
       audio track?
     */
     track_flag_t (*get_track_copy_permit) ( void *p_env, track_t i_track );
-  
-    /*!  
+
+    /*!
       Return the starting LBA for track number
       i_track in p_env.  Tracks numbers start at 1.
       The "leadout" track is specified either by
@@ -270,7 +303,7 @@ extern "C" {
     */
     lba_t (*get_track_lba) ( void *p_env, track_t i_track );
 
-    /*!  
+    /*!
       Return the starting LBA for the pregap for track number
       i_track in p_env.  Tracks numbers start at 1.
       CDIO_INVALID_LBA is returned on error.
@@ -285,23 +318,23 @@ extern "C" {
       string when done with it.
     */
     char * (*get_track_isrc) ( const void *p_env, track_t i_track );
-    
-    /*!  
-      Get format of track. 
+
+    /*!
+      Get format of track.
     */
     track_format_t (*get_track_format) ( void *p_env, track_t i_track );
-    
+
     /*!
       Return true if we have XA data (green, mode2 form1) or
       XA data (green, mode2 form2). That is track begins:
       sync - header - subheader
       12     4      -  8
-      
+
       FIXME: there's gotta be a better design for this and get_track_format?
     */
     bool (*get_track_green) ( void *p_env, track_t i_track );
-    
-    /*!  
+
+    /*!
       Return the starting MSF (minutes/secs/frames) for track number
       i_track in p_env.  Tracks numbers start at 1.
       The "leadout" track is specified either by
@@ -309,104 +342,104 @@ extern "C" {
       False is returned on error.
     */
     bool (*get_track_msf) ( void *p_env, track_t i_track, msf_t *p_msf );
-    
+
     /*! Return 1 if track has pre-emphasis, 0 if not, or -1 for error
       or -2 if not implimented (yet). Is this meaningful if not an
       audio track?
     */
-    track_flag_t (*get_track_preemphasis) 
+    track_flag_t (*get_track_preemphasis)
          ( const void  *p_env, track_t i_track );
-  
+
     /*!
       lseek - reposition read/write file offset
-      Returns (off_t) -1 on error. 
+      Returns (off_t) -1 on error.
       Similar to libc's lseek()
     */
     off_t (*lseek) ( void *p_env, off_t offset, int whence );
-    
+
     /*!
       Reads into buf the next size bytes.
-      Returns -1 on error. 
+      Returns -1 on error.
       Similar to libc's read()
     */
     ssize_t (*read) ( void *p_env, void *p_buf, size_t i_size );
-    
+
     /*!
       Reads a single mode2 sector from cd device into buf starting
-      from lsn. Returns 0 if no error. 
+      from lsn. Returns 0 if no error.
     */
     int (*read_audio_sectors) ( void *p_env, void *p_buf, lsn_t i_lsn,
                                 unsigned int i_blocks );
-    
+
     /*!
       Read a data sector
-      
+
       @param p_env environment to read from
 
       @param p_buf place to read data into.  The caller should make sure
-      this location can store at least CDIO_CD_FRAMESIZE, 
+      this location can store at least CDIO_CD_FRAMESIZE,
       M2RAW_SECTOR_SIZE, or M2F2_SECTOR_SIZE depending
-      on the kind of sector getting read. If you don't 
+      on the kind of sector getting read. If you don't
       know whether you have a Mode 1/2, Form 1/ Form 2/Formless
-      sector best to reserve space for the maximum, 
+      sector best to reserve space for the maximum,
       M2RAW_SECTOR_SIZE.
 
       @param i_lsn sector to read
-      @param i_blocksize size of block. Should be either CDIO_CD_FRAMESIZE, 
+      @param i_blocksize size of block. Should be either CDIO_CD_FRAMESIZE,
       M2RAW_SECTOR_SIZE, or M2F2_SECTOR_SIZE. See comment above under p_buf.
     */
-    driver_return_code_t (*read_data_sectors) 
+    driver_return_code_t (*read_data_sectors)
          ( void *p_env, void *p_buf, lsn_t i_lsn, uint16_t i_blocksize,
            uint32_t i_blocks );
-    
+
     /*!
       Reads a single mode2 sector from cd device into buf starting
-      from lsn. Returns 0 if no error. 
+      from lsn. Returns 0 if no error.
     */
-    int (*read_mode2_sector) 
+    int (*read_mode2_sector)
          ( void *p_env, void *p_buf, lsn_t i_lsn, bool b_mode2_form2 );
-    
+
     /*!
       Reads i_blocks of mode2 sectors from cd device into data starting
       from lsn.
-      Returns 0 if no error. 
+      Returns 0 if no error.
     */
-    int (*read_mode2_sectors) 
-         ( void *p_env, void *p_buf, lsn_t i_lsn, bool b_mode2_form2, 
+    int (*read_mode2_sectors)
+         ( void *p_env, void *p_buf, lsn_t i_lsn, bool b_mode2_form2,
            unsigned int i_blocks );
-    
+
     /*!
       Reads a single mode1 sector from cd device into buf starting
-      from lsn. Returns 0 if no error. 
+      from lsn. Returns 0 if no error.
     */
-    int (*read_mode1_sector) 
+    int (*read_mode1_sector)
          ( void *p_env, void *p_buf, lsn_t i_lsn, bool mode1_form2 );
-    
+
     /*!
       Reads i_blocks of mode1 sectors from cd device into data starting
       from lsn.
-      Returns 0 if no error. 
+      Returns 0 if no error.
     */
-    int (*read_mode1_sectors) 
-         ( void *p_env, void *p_buf, lsn_t i_lsn, bool mode1_form2, 
+    int (*read_mode1_sectors)
+         ( void *p_env, void *p_buf, lsn_t i_lsn, bool mode1_form2,
            unsigned int i_blocks );
-    
+
     bool (*read_toc) ( void *p_env ) ;
 
     /*!
-      Run a SCSI MMC command. 
-      
+      Run a SCSI MMC command.
+
       cdio              CD structure set by cdio_open().
       i_timeout_ms      time in milliseconds we will wait for the command
-                        to complete. 
+                        to complete.
       cdb_len           number of bytes in cdb (6, 10, or 12).
-      cdb               CDB bytes. All values that are needed should be set on 
-                        input. 
-      b_return_data     TRUE if the command expects data to be returned in 
+      cdb               CDB bytes. All values that are needed should be set on
+                        input.
+      b_return_data     TRUE if the command expects data to be returned in
                         the buffer
       len               Size of buffer
       buf               Buffer for data, both sending and receiving
-      
+
       Returns 0 if command completed successfully.
     */
     mmc_run_cmd_fn_t run_mmc_cmd;
@@ -415,16 +448,16 @@ extern "C" {
       Set the arg "key" with "value" in the source device.
     */
     int (*set_arg) ( void *p_env, const char key[], const char value[] );
-    
+
     /*!
-      Set the blocksize for subsequent reads. 
+      Set the blocksize for subsequent reads.
     */
-    driver_return_code_t (*set_blocksize) ( void *p_env, 
+    driver_return_code_t (*set_blocksize) ( void *p_env,
                                             uint16_t i_blocksize );
 
     /*!
-      Set the drive speed. 
-      
+      Set the drive speed.
+
       @return 0 if everything went okay, -1 if we had an error. is -2
       returned if this is not implemented for the current driver.
     */
@@ -441,51 +474,51 @@ extern "C" {
     void *env;             /**< environment. Passed to routine above. */
   };
 
-  /* This is used in drivers that must keep their own internal 
+  /* This is used in drivers that must keep their own internal
      position pointer for doing seeks. Stream-based drivers (like bincue,
-     nrg, toc, network) would use this. 
+     nrg, toc, network) would use this.
    */
-  typedef struct 
+  typedef struct
   {
     off_t   buff_offset;      /* buffer offset in disk-image seeks. */
     track_t index;            /* Current track index in tocent. */
     lba_t   lba;              /* Current LBA */
   } internal_position_t;
-  
+
   CdIo_t * cdio_new (generic_img_private_t *p_env, cdio_funcs_t *p_funcs);
 
   /* The below structure describes a specific CD Input driver  */
-  typedef struct 
+  typedef struct
   {
     driver_id_t  id;
     unsigned int flags;
     const char  *name;
     const char  *describe;
-    bool (*have_driver) (void); 
-    CdIo_t *(*driver_open) (const char *psz_source_name); 
-    CdIo_t *(*driver_open_am) (const char *psz_source_name, 
-                             const char *psz_access_mode); 
-    char *(*get_default_device) (void); 
+    bool (*have_driver) (void);
+    CdIo_t *(*driver_open) (const char *psz_source_name);
+    CdIo_t *(*driver_open_am) (const char *psz_source_name,
+                             const char *psz_access_mode);
+    char *(*get_default_device) (void);
     bool (*is_device) (const char *psz_source_name);
     char **(*get_devices) (void);
     driver_return_code_t (*close_tray) (const char *psz_device);
   } CdIo_driver_t;
 
-  /* The below array gives of the drivers that are currently available for 
+  /* The below array gives of the drivers that are currently available for
      on a particular host. */
   extern CdIo_driver_t CdIo_driver[];
 
-  /* The last valid entry of Cdio_driver. -1 means uninitialzed. -2 
+  /* The last valid entry of Cdio_driver. -1 means uninitialzed. -2
      means some sort of error.
    */
-  extern int CdIo_last_driver; 
+  extern int CdIo_last_driver;
 
   /* The below array gives all drivers that can possibly appear.
      on a particular host. */
   extern CdIo_driver_t CdIo_all_drivers[];
 
-  /*! 
-    Add/allocate a drive to the end of drives. 
+  /*!
+    Add/allocate a drive to the end of drives.
     Use cdio_free_device_list() to free this device_list.
   */
   void cdio_add_device_list(char **device_list[], const char *psz_drive,
@@ -495,7 +528,6 @@ extern "C" {
   driver_return_code_t close_tray_freebsd (const char *psz_drive);
   driver_return_code_t close_tray_linux   (const char *psz_drive);
   driver_return_code_t close_tray_netbsd  (const char *psz_drive);
-  driver_return_code_t close_tray_os2     (const char *psz_drive);
   driver_return_code_t close_tray_osx     (const char *psz_drive);
   driver_return_code_t close_tray_solaris (const char *psz_drive);
   driver_return_code_t close_tray_win32   (const char *psz_drive);
