@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2004, 2005, 2008, 2011, 2012 Rocky Bernstein <rocky@gnu.org>
+  Copyright (C) 2004-2005, 2008, 2011, 2012, 2013 Rocky Bernstein <rocky@gnu.org>
   toc reading routine adapted from cuetools
   Copyright (C) 2003 Svend Sanjay Sorensen <ssorensen@fastmail.fm>
 
@@ -39,11 +39,7 @@
 #define MAX_CDTEXT_GENRE_CODE     28
 #define MAX_CDTEXT_LANGUAGE_CODE 127
 
-#ifndef CDIO_CD_MAX_TRACKS
-# define CDIO_CD_MAX_TRACKS 99 /* Largest CD track number */
-#endif
-
-const char *cdtext_field[MAX_CDTEXT_FIELDS] = 
+const char *cdtext_field[MAX_CDTEXT_FIELDS] =
 {
   "TITLE",
   "PERFORMER",
@@ -205,7 +201,7 @@ cdtext_field2str(cdtext_field_t i)
 {
   if (i >= MAX_CDTEXT_FIELDS)
     return "INVALID";
-  else 
+  else
     return cdtext_field[i];
 }
 
@@ -236,16 +232,16 @@ cdtext_lang2str(cdtext_lang_t i)
 /*!
   Free memory associated with the given cdtext_t object.
 
-  @param p_cdtext the CD-TEXT object 
+  @param p_cdtext the CD-TEXT object
 */
-void 
+void
 cdtext_destroy(cdtext_t *p_cdtext)
 {
   cdtext_field_t k;
   track_t j;
   int i;
 
-  if (!p_cdtext) return; 
+  if (!p_cdtext) return;
   for (i=0; i<CDTEXT_NUM_BLOCKS_MAX; i++) {
     for (j=0; j<CDTEXT_NUM_TRACKS_MAX; j++) {
       for (k=0; k < MAX_CDTEXT_FIELDS; k++) {
@@ -256,12 +252,13 @@ cdtext_destroy(cdtext_t *p_cdtext)
       }
     }
   }
+  free(p_cdtext);
 }
 
 /*!
   Returns a copy of the return value of cdtext_get_const or NULL.
 
-  Should be freed when done.
+  Must be freed using cdio_free() when done.
   @see cdtext_get_const
 */
 char *
@@ -289,7 +286,6 @@ cdtext_get_const(const cdtext_t *p_cdtext, cdtext_field_t field, track_t track)
 {
   if (CDTEXT_FIELD_INVALID == field
       || NULL == p_cdtext
-      || 0 > track
       || CDIO_CD_MAX_TRACKS < track)
     return NULL;
 
@@ -322,6 +318,32 @@ cdtext_get_language(const cdtext_t *p_cdtext)
   return p_cdtext->block[p_cdtext->block_i].language_code;
 }
 
+/*!
+  Returns the first track number.
+
+  @param p_cdtext the CD-TEXT object
+*/
+track_t
+cdtext_get_first_track(const cdtext_t *p_cdtext)
+{
+  if (NULL == p_cdtext)
+    return 0;
+  return p_cdtext->block[p_cdtext->block_i].first_track;
+}
+
+/*!
+  Returns the last track number.
+
+  @param p_cdtext the CD-TEXT object
+*/
+track_t
+cdtext_get_last_track(const cdtext_t *p_cdtext)
+{
+  if (NULL == p_cdtext)
+    return 0;
+  return p_cdtext->block[p_cdtext->block_i].last_track;
+}
+
 /*
   Returns a list of available languages or NULL.
 
@@ -338,7 +360,7 @@ cdtext_lang_t
   if (NULL == p_cdtext)
     return NULL;
 
-  for (i=0; i<CDTEXT_NUM_BLOCKS_MAX; i++) 
+  for (i=0; i<CDTEXT_NUM_BLOCKS_MAX; i++)
   {
     avail[i] = CDTEXT_LANGUAGE_UNKNOWN;
     if (CDTEXT_LANGUAGE_UNKNOWN != p_cdtext->block[i].language_code)
@@ -379,14 +401,14 @@ cdtext_select_language(cdtext_t *p_cdtext, cdtext_lang_t language)
   return false;
 }
 
-/*! 
+/*!
   Initialize a new cdtext structure.
 
-  When the structure is no longer needed, release the 
+  When the structure is no longer needed, release the
   resources using cdtext_delete.
 
 */
-cdtext_t 
+cdtext_t
 *cdtext_init(void)
 {
   cdtext_field_t k;
@@ -417,14 +439,14 @@ cdtext_t
   Internal function.
 
   @param key key to test
-  
+
   @return CDTEXT_INVALID if the given keyword is invalid
 */
 cdtext_field_t
 cdtext_is_field (const char *key)
 {
   unsigned int i;
-  
+
   for (i = 0; i < MAX_CDTEXT_FIELDS ; i++)
     if (0 == strcmp(cdtext_field[i], key)) {
       return i;
@@ -438,14 +460,14 @@ cdtext_is_field (const char *key)
   Internal function.
 
   @param lang language to test
-  
+
   @return CDTEXT_LANGUAGE_UNKNOWN if language is not supported
 */
 cdtext_lang_t
 cdtext_is_language(const char *lang)
 {
   unsigned int i;
-  
+
   for (i = 0; i < MAX_CDTEXT_LANGUAGE_CODE; i++)
     if (0 == strcmp(cdtext_language[i], lang)) {
       return i;
@@ -453,39 +475,41 @@ cdtext_is_language(const char *lang)
   return CDTEXT_LANGUAGE_UNKNOWN;
 }
 
-/*! 
+/*!
   Sets the given field at the given track to the given value.
-  
+
   Recodes to UTF-8 if charset is not NULL.
-  
+
   @param p_cdtext the CD-TEXT object
   @param key field to set
   @param value value to set
   @param track track to work on
   @param charset charset to convert from
  */
-void 
-cdtext_set(cdtext_t *p_cdtext, cdtext_field_t key, const uint8_t *value, 
+void
+cdtext_set(cdtext_t *p_cdtext, cdtext_field_t key, const uint8_t *value,
            track_t track, const char *charset)
 {
-  if (NULL == value || key == CDTEXT_FIELD_INVALID || 0 > track 
+  if (NULL == value || key == CDTEXT_FIELD_INVALID
       || CDIO_CD_MAX_TRACKS < track)
     return;
 
   /* free old memory */
-  if (p_cdtext->block[p_cdtext->block_i].track[track].field[key]) 
+  if (p_cdtext->block[p_cdtext->block_i].track[track].field[key])
     free(p_cdtext->block[p_cdtext->block_i].track[track].field[key]);
 
   /* recode to UTF-8 */
   if (NULL != charset) {
     cdio_utf8_t *utf8_str = NULL;
-    cdio_charset_to_utf8((const char*) value, strlen((const char*)value), 
+    cdio_charset_to_utf8((const char*) value, strlen((const char*)value),
                         &utf8_str, charset);
     p_cdtext->block[p_cdtext->block_i].track[track].field[key] = (char *)utf8_str;
   } else
     p_cdtext->block[p_cdtext->block_i].track[track].field[key] = strdup((const char *)value);
 }
 
+#define CDTEXT_COMPARE_CHAR(buf, c, db) ((buf)[0] == c && (! db || (buf)[1] == c) )
+
 /*!
   Read a binary CD-TEXT and fill a cdtext struct.
 
@@ -494,21 +518,24 @@ cdtext_set(cdtext_t *p_cdtext, cdtext_field_t key, const uint8_t *value,
   @param i_data size of wdata
 
   @returns 0 on success, non-zero on failure
-*/       
+*/
 int
-cdtext_data_init(cdtext_t *p_cdtext, uint8_t *wdata, size_t i_data) 
+cdtext_data_init(cdtext_t *p_cdtext, uint8_t *wdata, size_t i_data)
 {
   uint8_t       *p_data;
   int           j;
   uint8_t       buffer[256];
+  uint8_t       tab_buffer[256];
   int           i_buf = 0;
   int           i_block;
   int           i_seq = 0;
   int           i;
   cdtext_blocksize_t blocksize;
   char          *charset = NULL;
+  uint8_t       cur_track;
 
   memset( buffer, 0, sizeof(buffer) );
+  memset( tab_buffer, 0, sizeof(buffer) );
 
   p_data = wdata;
   if (i_data < CDTEXT_LEN_PACK || 0 != i_data % CDTEXT_LEN_PACK) {
@@ -613,6 +640,11 @@ cdtext_data_init(cdtext_t *p_cdtext, uint8_t *wdata, size_t i_data)
             charset = (char *) "SHIFT_JIS";
             break;
         }
+
+        /* set track numbers */
+        p_cdtext->block[i_block].first_track = blocksize.i_first_track;
+        p_cdtext->block[i_block].last_track = blocksize.i_last_track;
+
       } else {
         cdio_warn("CD-TEXT: No blocksize information available for block %d.\n", i_block);
         return -1;
@@ -629,6 +661,8 @@ cdtext_data_init(cdtext_t *p_cdtext, uint8_t *wdata, size_t i_data)
     }
 #endif
 
+    cur_track = pack.i_track;
+
     /* read text packs first */
     j = 0;
     switch (pack.type) {
@@ -651,49 +685,72 @@ cdtext_data_init(cdtext_t *p_cdtext, uint8_t *wdata, size_t i_data)
       case CDTEXT_PACK_UPC:
         while (j < CDTEXT_LEN_TEXTDATA) {
           /* not terminated */
-          if (pack.text[j] != 0 || (pack.db_chars && pack.text[j+1] != 0)) {
+
+          if ( i_buf+2 >= sizeof(buffer)) {
+            cdio_warn("CD-TEXT: Field too long.");
+            return -1;
+          }
+
+          /* if the first character is a TAB, copy the buffer */
+          if ( i_buf == 0 && CDTEXT_COMPARE_CHAR(&pack.text[j], '\t', pack.db_chars)) {
+            memcpy(tab_buffer, buffer, sizeof(tab_buffer));
+          }
+
+          if ( ! CDTEXT_COMPARE_CHAR(&pack.text[j], '\0', pack.db_chars)) {
             buffer[i_buf++] = pack.text[j];
             if(pack.db_chars)
               buffer[i_buf++] = pack.text[j+1];
-          } else if(i_buf > 1) {
-            buffer[i_buf++] = 0;
-            if(pack.db_chars)
+          } else if(i_buf > 0) {
+            /* if end of string */
+
+            /* check if the buffer contains only the Tab Indicator */
+            if ( CDTEXT_COMPARE_CHAR(buffer, '\t', pack.db_chars) ) {
+              if ( cur_track <= blocksize.i_first_track ) {
+                cdio_warn("CD-TEXT: Invalid use of Tab Indicator.");
+                return -1;
+              }
+              memcpy(buffer, tab_buffer, sizeof(buffer));
+            } else {
               buffer[i_buf++] = 0;
+              if(pack.db_chars)
+                buffer[i_buf++] = 0;
+            }
 
             switch (pack.type) {
               case CDTEXT_PACK_TITLE:
-                cdtext_set(p_cdtext, CDTEXT_FIELD_TITLE, buffer, pack.i_track, charset);
+                cdtext_set(p_cdtext, CDTEXT_FIELD_TITLE, buffer, cur_track, charset);
                 break;
               case CDTEXT_PACK_PERFORMER:
-                cdtext_set(p_cdtext, CDTEXT_FIELD_PERFORMER, buffer, pack.i_track, charset);
+                cdtext_set(p_cdtext, CDTEXT_FIELD_PERFORMER, buffer, cur_track, charset);
                 break;
               case CDTEXT_PACK_SONGWRITER:
-                cdtext_set(p_cdtext, CDTEXT_FIELD_SONGWRITER, buffer, pack.i_track, charset);
+                cdtext_set(p_cdtext, CDTEXT_FIELD_SONGWRITER, buffer, cur_track, charset);
                 break;
               case CDTEXT_PACK_COMPOSER:
-                cdtext_set(p_cdtext, CDTEXT_FIELD_COMPOSER, buffer, pack.i_track, charset);
+                cdtext_set(p_cdtext, CDTEXT_FIELD_COMPOSER, buffer, cur_track, charset);
                 break;
               case CDTEXT_PACK_ARRANGER:
-                cdtext_set(p_cdtext, CDTEXT_FIELD_ARRANGER, buffer, pack.i_track, charset);
+                cdtext_set(p_cdtext, CDTEXT_FIELD_ARRANGER, buffer, cur_track, charset);
                 break;
               case CDTEXT_PACK_MESSAGE:
-                cdtext_set(p_cdtext, CDTEXT_FIELD_MESSAGE, buffer, pack.i_track, charset);
+                cdtext_set(p_cdtext, CDTEXT_FIELD_MESSAGE, buffer, cur_track, charset);
                 break;
               case CDTEXT_PACK_DISCID:
-                if (pack.i_track == 0)
-                  cdtext_set(p_cdtext, CDTEXT_FIELD_DISCID, buffer, pack.i_track, NULL);
+                if (cur_track == 0)
+                  cdtext_set(p_cdtext, CDTEXT_FIELD_DISCID, buffer, cur_track, NULL);
                 break;
               case CDTEXT_PACK_GENRE:
-                cdtext_set(p_cdtext, CDTEXT_FIELD_GENRE, buffer, pack.i_track, "ASCII");
+                cdtext_set(p_cdtext, CDTEXT_FIELD_GENRE, buffer, cur_track, "ASCII");
                 break;
               case CDTEXT_PACK_UPC:
-                if (pack.i_track == 0)
-                  cdtext_set(p_cdtext, CDTEXT_FIELD_UPC_EAN, buffer, pack.i_track, "ASCII");
+                if (cur_track == 0)
+                  cdtext_set(p_cdtext, CDTEXT_FIELD_UPC_EAN, buffer, cur_track, "ASCII");
                 else
-                  cdtext_set(p_cdtext, CDTEXT_FIELD_ISRC, buffer, pack.i_track, "ISO-8859-1");
+                  cdtext_set(p_cdtext, CDTEXT_FIELD_ISRC, buffer, cur_track, "ISO-8859-1");
                 break;
             }
             i_buf = 0;
+            ++cur_track;
 
           }
           if (pack.db_chars)
