@@ -25,13 +25,13 @@ fi
 
 BASE=`basename $0 .sh`
 test_name=videocd
-opts="--quiet --no-device-info --nrg-file ${srcdir}/${test_name}.nrg $vcd_opt --iso9660"
+opts="--quiet --no-device-info --nrg-file ${srcdir}/data/${test_name}.nrg $vcd_opt --iso9660"
 test_cdinfo "$opts" ${test_name}.dump ${srcdir}/${test_name}.right
 RC=$?
 check_result $RC 'cd-info NRG test 1' "${CD_INFO} $opts"
 
 BASE=`basename $0 .sh`
-nrg_file=${srcdir}/monvoisin.nrg
+nrg_file=${srcdir}/data/monvoisin.nrg
 
 if test -f  $nrg_file ; then
   test_cdinfo "-q --no-device-info --nrg-file $nrg_file $vcd_opt --iso9660 " \
@@ -39,24 +39,24 @@ if test -f  $nrg_file ; then
   RC=$?
   check_result $RC 'cd-info NRG test 2'
 else 
-  echo "Don't see NRG file ${nrg_file}. Test skipped."
+  echo "-- Don't see NRG file ${nrg_file}. Test skipped."
   exit 0
 fi
 
 test_name='svcdgs'
-nrg_file=${srcdir}/${test_name}.nrg
+nrg_file=${srcdir}/data/${test_name}.nrg
 opts="-q --no-device-info --nrg-file $nrg_file $vcd_opt --iso9660"
 if test -f  $nrg_file ; then
   test_cdinfo "$opts" ${test_name}.dump ${srcdir}/${test_name}.right
   RC=$?
   check_result $RC "cd-info NRG $test_name" "${CD_INFO} $opts"
 else 
-  echo "Don't see NRG file ${nrg_file}. Test skipped."
+  echo "-- Don't see NRG file ${nrg_file}. Test skipped."
   exit $SKIP_TEST_EXITCODE
 fi
 
 test_name='cdda-mcn'
-nrg_file=${srcdir}/${test_name}.nrg
+nrg_file=${srcdir}/data/${test_name}.nrg
 opts="-q --no-device-info --nrg-file $nrg_file --no-cddb"
 if test -f  $nrg_file ; then
   test_cdinfo "$opts" ${test_name}.dump ${srcdir}/${test_name}.right
@@ -64,7 +64,7 @@ if test -f  $nrg_file ; then
   check_result $RC "cd-info NRG $test_name" "${CD_INFO} $opts"
   exit $RC
 else 
-  echo "Don't see NRG file ${nrg_file}. Test skipped."
+  echo "-- Don't see NRG file ${nrg_file}. Test skipped."
   exit $SKIP_TEST_EXITCODE
 fi
 
